@@ -28,6 +28,7 @@ from jinja2.exceptions import TemplateRuntimeError
 from mock import Mock, patch
 import os
 import renderspec
+import renderspec.contextfuncs
 import renderspec.utils
 import renderspec.versions
 import shutil
@@ -40,12 +41,12 @@ class RenderspecContextFunctionTests(unittest.TestCase):
     """test functions which do some calculation based on the context"""
     def test_context_license_spdx(self):
         self.assertEqual(
-            renderspec._context_license_spdx(
+            renderspec.contextfuncs._context_license_spdx(
                 {'spec_style': 'suse'}, 'Apache-2.0'),
             'Apache-2.0'
         )
         self.assertEqual(
-            renderspec._context_license_spdx(
+            renderspec.contextfuncs._context_license_spdx(
                 {'spec_style': 'fedora'}, 'Apache-2.0'),
             'ASL 2.0'
         )
@@ -53,62 +54,86 @@ class RenderspecContextFunctionTests(unittest.TestCase):
     @data(
         # without version
         ({'spec_style': 'suse', 'epochs': {}, 'requirements': {}},
-         'oslo.config', None, 'python-oslo.config'),
+         'oslo.config', None, None, 'python-oslo.config'),
         ({'spec_style': 'fedora', 'epochs': {}, 'requirements': {}},
-         'oslo.config', None, 'python-oslo-config'),
+         'oslo.config', None, None, 'python-oslo-config'),
+        # without version, multiple python versions
+        ({'spec_style': 'suse', 'epochs': {}, 'requirements': {}},
+         'oslo.config', None, ('py', 'py3'),
+         'python-oslo.config python3-oslo.config'),
         # with version
         ({'spec_style': 'suse', 'epochs': {}, 'requirements': {}},
-         'oslo.config', ('>=', '1.2.3'), 'python-oslo.config >= 1.2.3'),
+         'oslo.config', ('>=', '1.2.3'), None, 'python-oslo.config >= 1.2.3'),
         ({'spec_style': 'fedora', 'epochs': {}, 'requirements': {}},
-         'oslo.config', ('==', '1.2.3~a0'), 'python-oslo-config == 1.2.3~a0'),
+         'oslo.config', ('==', '1.2.3~a0'), None,
+         'python-oslo-config == 1.2.3~a0'),
         # with version, with epoch
         ({'spec_style': 'suse', 'epochs': {'oslo.config': 4},
           'requirements': {}},
-         'oslo.config', ('>=', '1.2.3'), 'python-oslo.config >= 4:1.2.3'),
+         'oslo.config', ('>=', '1.2.3'), None,
+         'python-oslo.config >= 4:1.2.3'),
         # without version, with epoch
         ({'spec_style': 'suse', 'epochs': {'oslo.config': 4},
           'requirements': {}},
-         'oslo.config', None, 'python-oslo.config'),
+         'oslo.config', None, None, 'python-oslo.config'),
         # with version, with requirements
         ({'spec_style': 'suse', 'epochs': {},
           'requirements': {'oslo.config' '1.2.3'}},
-         'oslo.config', ('>=', '4.5.6'), 'python-oslo.config >= 4.5.6'),
+         'oslo.config', ('>=', '4.5.6'), None, 'python-oslo.config >= 4.5.6'),
         # without version, with requirements
         ({'spec_style': 'suse', 'epochs': {},
           'requirements': {'oslo.config': '1.2.3'}},
-         'oslo.config', None, 'python-oslo.config >= 1.2.3'),
+         'oslo.config', None, None, 'python-oslo.config >= 1.2.3'),
         # without version, with requirements, with epoch
         ({'spec_style': 'suse', 'epochs': {'oslo.config': 4},
           'requirements': {'oslo.config': '1.2.3'}},
-         'oslo.config', None, 'python-oslo.config >= 4:1.2.3'),
+         'oslo.config', None, None, 'python-oslo.config >= 4:1.2.3'),
         # with version, with requirements, with epoch
         ({'spec_style': 'suse', 'epochs': {'oslo.config': 4},
           'requirements': {'oslo.config' '1.2.3'}},
-         'oslo.config', ('>=', '4.5.6'), 'python-oslo.config >= 4:4.5.6'),
+         'oslo.config', ('>=', '4.5.6'), None,
+         'python-oslo.config >= 4:4.5.6'),
+        # with version, with requirements, with epoch, python2
+        ({'spec_style': 'suse', 'epochs': {'oslo.config': 4},
+          'requirements': {'oslo.config' '1.2.3'}},
+         'oslo.config', ('>=', '4.5.6'), 'py2',
+         'python2-oslo.config >= 4:4.5.6'),
+        # with version, with requirements, with epoch, python3
+        ({'spec_style': 'suse', 'epochs': {'oslo.config': 4},
+          'requirements': {'oslo.config' '1.2.3'}},
+         'oslo.config', ('>=', '4.5.6'), 'py3',
+         'python3-oslo.config >= 4:4.5.6'),
+        # with version, with requirements, with epoch, python2 and python3
+        ({'spec_style': 'suse', 'epochs': {'oslo.config': 4},
+          'requirements': {'oslo.config' '1.2.3'}},
+         'oslo.config', ('>=', '4.5.6'), ['py2', 'py3'],
+         'python2-oslo.config >= 4:4.5.6 python3-oslo.config >= 4:4.5.6'),
+
     )
     @unpack
     def test_context_py2pkg(self, context, pkg_name, pkg_version,
-                            expected_result):
+                            py_versions, expected_result):
         self.assertEqual(
-            renderspec._context_py2pkg(context, pkg_name, pkg_version),
+            renderspec.contextfuncs._context_py2pkg(
+                context, pkg_name, pkg_version, py_versions),
             expected_result)
 
     def test_context_epoch_without_epochs(self):
         self.assertEqual(
-            renderspec._context_epoch(
+            renderspec.contextfuncs._context_epoch(
                 {'spec_style': 'suse', 'epochs': {}, 'requirements': {}},
                 'oslo.config'), 0)
 
     def test_context_epoch_with_epochs(self):
         self.assertEqual(
-            renderspec._context_epoch(
+            renderspec.contextfuncs._context_epoch(
                 {'spec_style': 'suse', 'epochs': {'oslo.config': 4},
                  'requirements': {}}, 'oslo.config'), 4)
 
     def test_context_upstream_version(self):
         context = {'spec_style': 'suse', 'epochs': {},
                    'requirements': {}}
-        self.assertEqual(renderspec._context_upstream_version(
+        self.assertEqual(renderspec.contextfuncs._context_upstream_version(
                 context, '1.2.0'), '1.2.0')
 
     @data(
@@ -124,7 +149,7 @@ class RenderspecContextFunctionTests(unittest.TestCase):
                    'requirements': {}, 'output_dir': output_dir}
         url = 'http://foo/bar'
         with patch('renderspec.utils._download_file') as m:
-            self.assertEqual(renderspec._context_fetch_source(
+            self.assertEqual(renderspec.contextfuncs._context_fetch_source(
                 context, url), url)
             self.assertEqual(m.call_count, expected_calls)
 
@@ -134,7 +159,7 @@ class RenderspecTemplateFunctionTests(unittest.TestCase):
     def setUp(self):
         """create a Jinja2 environment and register the standard filters"""
         self.env = Environment()
-        renderspec._env_register_filters_and_globals(self.env)
+        renderspec.contextfuncs.env_register_filters_and_globals(self.env)
 
     @data(
         ("{{ 'http://foo/bar'|basename }}", "bar")
@@ -157,6 +182,10 @@ class RenderspecTemplateFunctionTests(unittest.TestCase):
         # plain
         ({'spec_style': 'suse', 'epochs': {}, 'requirements': {}},
          "{{ py2pkg('requests') }}", "python-requests"),
+        # plain, with multiple py_versions
+        ({'spec_style': 'suse', 'epochs': {}, 'requirements': {}},
+         "{{ py2pkg('requests', py_versions=['py2', 'py3']) }}",
+         "python2-requests python3-requests"),
         # with version
         ({'spec_style': 'suse', 'epochs': {}, 'requirements': {}},
          "{{ py2pkg('requests', ('>=', '2.8.1')) }}",
@@ -175,6 +204,11 @@ class RenderspecTemplateFunctionTests(unittest.TestCase):
           'requirements': {'requests': '1.2.3'}},
          "{{ py2pkg('requests') }}",
          "python-requests >= 4:1.2.3"),
+        # without version, with epoch, with requirements, with py_versions
+        ({'spec_style': 'suse', 'epochs': {'requests': 4},
+          'requirements': {'requests': '1.2.3'}},
+         "{{ py2pkg('requests', py_versions=['py2']) }}",
+         "python2-requests >= 4:1.2.3"),
     )
     @unpack
     def test_render_func_py2pkg(self, context, string, expected_result):
@@ -246,8 +280,8 @@ class RenderspecTemplateFunctionTests(unittest.TestCase):
         ('suse', '1.1.0a10', '1.1.0~xalpha10'),
         ('suse', '1.1.0a10dev5', '1.1.0~xalpha10~dev5'),
         ('suse', '1.1.0b10', '1.1.0~xbeta10'),
-        ('suse', '1.1.0rc2', '1.1.0~rc2'),
-        ('suse', '1.1.0rc2dev2', '1.1.0~rc2~dev2'),
+        ('suse', '1.1.0rc2', '1.1.0~xrc2'),
+        ('suse', '1.1.0rc2dev2', '1.1.0~xrc2~dev2'),
         ('fedora', '1.1.0', '1.1.0'),
         ('fedora', '1.1.0b10', '1.1.0'),
         ('fedora', '1.1.0rc2dev2', '1.1.0'),
@@ -291,6 +325,18 @@ class RenderspecTemplateFunctionTests(unittest.TestCase):
             template.render(**context),
             rpm_release_expected)
 
+    def test_render_func_url_pypi(self):
+        context = {'spec_style': 'suse', 'epochs': {}, 'requirements': {}}
+        # need to escape '{' and '}' here
+        s = "{% set upstream_version = '3.20.0' %}" \
+            "{% set pypi_name = 'oslo.concurrency' %}" \
+            "{{ url_pypi() }}"
+        template = self.env.from_string(s)
+        self.assertEqual(
+            template.render(**context),
+            "https://files.pythonhosted.org/packages/source/o/"
+            "oslo.concurrency/oslo.concurrency-3.20.0.tar.gz")
+
 
 class RenderspecVersionsTests(unittest.TestCase):
     def test_without_version(self):
@@ -375,7 +421,7 @@ class RenderspecCommonTests(unittest.TestCase):
             with open(f1, 'w+') as f:
                 f.write(template)
             rendered = renderspec.generate_spec(
-                style, epochs, requirements, f1, None)
+                style, epochs, requirements, 'spec.j2', f1, None)
             self.assertEqual(rendered, expected_result)
         finally:
             shutil.rmtree(tmpdir)
@@ -438,8 +484,8 @@ class RenderspecDistTeamplatesTests(unittest.TestCase):
             # mock this to use testing dist-tempaltes folder
             mock_dt_path.return_value = dt_dir
 
-            out = renderspec.generate_spec('loldistro', {}, {}, base_path,
-                                           None)
+            out = renderspec.generate_spec('loldistro', {}, {}, 'spec.j2',
+                                           base_path, None)
             self.assertEqual(out, expected_out)
         finally:
             shutil.rmtree(tmpdir)
